<?php

/**
 * The public-facing functionality of Visidea.
 *
 * @link       https://visidea.ai
 * @since      1.0.0
 *
 * @package    Visidea
 * @subpackage Visidea/public
 */

class Visidea_Public {

  /**
   * The ID of Visidea.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $plugin_name    The ID of this plugin.
   */
  private $plugin_name;

  /**
   * The version of Visidea.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $version    The current version of this plugin.
   */
  private $version;

  /**
   * Initialize the class and set its properties.
   *
   * @since    1.0.0
   * @param      string    $plugin_name       The name of the plugin.
   * @param      string    $version    The version of this plugin.
   */
  public function __construct( $plugin_name, $version ) {

  	$this->plugin_name = $plugin_name;
  	$this->version = $version;

  }

  /**
   * Register the stylesheets for the public-facing side of the site.
   *
   * @since 1.0.0
   */
  public function enqueue_styles() {

    wp_register_style('visidea_slick', plugin_dir_url( __FILE__ ) . 'css/slick.css', array(), $this->version, 'all');
    wp_register_style('visidea_slick_theme', plugin_dir_url( __FILE__ ) . 'css/slick-theme.css', array(), $this->version, 'all');
    wp_register_style('visidea_vs', 'https://cdn.visidea.ai/visual-search/css/main.css', array(), $this->version, 'all');

    wp_enqueue_style('visidea_slick');
    wp_enqueue_style('visidea_slick_theme');
    wp_enqueue_style('visidea_vs');
      
  }

  /**
   * Register the JavaScript for the public-facing side of the site.
   *
   * @since 1.0.0
   */
  public function enqueue_scripts() {

    wp_register_script('visidea_slick', plugin_dir_url( __FILE__ ) . 'js/slick.min.js', array('jquery'), $this->version, true);
    wp_register_script('visidea_api_client', 'https://cdn.visidea.ai/js-api-client/dist/visidea-api-client.min.js', array('jquery'), $this->version, true);
    wp_register_script('visidea_client', plugin_dir_url( __FILE__ ) . 'js/visidea-client.js', array('jquery'), $this->version, true);

    wp_enqueue_script('visidea_slick');
    wp_enqueue_script('visidea_api_client');
    wp_enqueue_script('visidea_client');

    // Initialize VisideaWoocommerce object
    $options = get_option('visidea_plugin_options');
    $visidea_public_token = $options['public_token'];
    $visidea_database = str_replace('.','_',$options['database']);
    wp_add_inline_script('visidea_client', "
      var vs_timer_start = window.setInterval(function(){
        if (typeof VisideaWoocommerce === 'function') {
            window.clearInterval(vs_timer_start);
            window.visideaWoocommerce = new VisideaWoocommerce('".esc_js($visidea_database)."','".esc_js($visidea_public_token)."');
          }
      }, 100);
    ");

  }

  /**
   * Action to execute when an order is placed.
   * Append to the interaction file (filename defined in activator)
   * the string user_id;purchase;item_id for each item in the
   * placed order.
   *
   * @since 1.0.0
   * @param int $order_id identificator of the placed order
   */
  public function callback_order_placed($order_id) {

    // the order and the customer_id
    // $query = new WC_Order_Query( array(
    //   'limit' => -1,
    //   'orderby' => 'date',
    //   'order' => 'DESC',
    // ));
    // var_dump($order_id);
    $order = new WC_Order( $order_id );
    // $order = get_order($order_id);
    // var_dump($order);
    $data  = $order->get_data();
    $customer_id = $order->get_customer_id();
    // var_dump($customer_id);
    // $customer_id = $this->vs_get_customer_id($order[0]->get_billing_email());

    if ($customer_id > 0) {
      echo "<script> 
      var vs_timer_order = window.setInterval(function(){
        if (window.visideaWoocommerce != undefined) {
            window.clearInterval(vs_timer_order);
            visideaWoocommerce.mergeUser('".esc_js($customer_id)."');
        }
      }, 100);
      </script>";  
    } else {
      // Loop through order items and write them in the opened file
      $product_ids = array();
      foreach ($order->get_items() as $item_id => $item) {
        // Get product handle
        $product = $item->get_product();

        // Get the product ID
        if (is_object($product))
          $product_id = $product->get_id();
        else
          $product_id = 0;
        if ($product_id > 0) {
          $product_ids[] = $product_id;
        }
      }
      if (count($product_ids))
        echo "<script>
          var vs_timer_order = window.setInterval(function(){
            if (window.visideaWoocommerce != undefined) {
                window.clearInterval(vs_timer_order);
                visideaWoocommerce.interaction('purchase', 'purchase', '".esc_js(get_current_user_id())."', ".esc_js(json_encode($product_ids)).");
            }
          }, 100);
        </script>";

    }

  }


  /**
   * Function that retrieve the customer_id from an e-mail.
   *
   * @since 1.3.2
   * @param string $email email of the customer
   */
  function vs_get_customer_id($email) {
    global $wpdb;
    $sql = "SELECT customer_id FROM {$wpdb->prefix}wc_customer_lookup WHERE email = '$email'";
    $customer = $wpdb->get_results($sql, OBJECT);
    return $customer[0]->customer_id;
  }


  /**
   *    Action to be performed when a product is addedd to a
   *    user's cart. Append the string item_id;cart;user_id
   *    to the file specified in the activator.
   *
   *    @since 1.0.0
   *    @param int      $cart_item_key          cart id
   *    @param int      $product_id             product id
   *    @param int      $quantity
   *    @param int      $variation_id           variation of product id
   *    @param int      $variation
   *    @param ?        $cart_item_data
   */
  public function callback_product_added_cart($cart_item_key, $product_id, $quantity,
    $variation_id, $variation, $cart_item_data ) {

    if (!wp_doing_ajax()) {
      $user_id = get_current_user_id();

      // Get handle to Visidea API
      $options = get_option('visidea_plugin_options');
      $visidea_public_token = $options['public_token'];
      $visidea_database = $options['database'];
  
      $item_array = [$product_id];
  
      // Use VisideaWoocommerce to send purchase messages
      echo "<script> 
      var vs_timer_cart = window.setInterval(function(){
        if (window.visideaWoocommerce != undefined) {
            window.clearInterval(vs_timer_cart);
            visideaWoocommerce.interaction('cart', 'cart', '".esc_js($user_id)."', ".esc_js(json_encode($item_array)).");
        }
      }, 100);
      </>";
    }

  }

  public function callback_home_viewed() {
      if (is_front_page()) {
          $user_id = get_current_user_id();

          // echo 'after content';
          $options = get_option('visidea_plugin_options');
          $visidea_public_token = $options['public_token'];
          $visidea_database = $options['database'];

          // Show recommendations
          echo "<script> 
          var vs_timer_home = window.setInterval(function(){
            if (window.visideaWoocommerce != undefined) {
                window.clearInterval(vs_timer_home);
                visideaWoocommerce.recommend('home');
            }
          }, 100);
          </script>";
      }
  }

  /**
   *  Action to be performed when a product page is loaded.
   *  Append to the file defined in the activator
   *  the string product_id;view;user_id. Since there's no way
   *  to perform this action _only the first time_ it's viewed,
   *  it will be repeated every time the user views a product page.
   */
  public function callback_product_viewed() {
    global $product;
    $product_id = $product->get_id();
    $user_id = get_current_user_id();

    // Get handle to Visidea API
    $options = get_option('visidea_plugin_options');
    $visidea_public_token = $options['public_token'];
    $visidea_database = $options['database'];

    $item_array = [$product_id];
    // Use VisideaWoocommerce to send product view messages
    echo "<script> 
    var vs_timer_product_1 = window.setInterval(function(){
      if (window.visideaWoocommerce != undefined) {
          window.clearInterval(vs_timer_product_1);
          visideaWoocommerce.interaction('product', 'view', '".esc_js($user_id)."', ".esc_js(json_encode($item_array)).");
      }
    }, 100);
    </script>";

    // Show recommendations
    echo "<script> 
    var vs_timer_product_2 = window.setInterval(function(){
      if (window.visideaWoocommerce != undefined) {
          window.clearInterval(vs_timer_product_2);
          visideaWoocommerce.recommend('product');
      }
    }, 100);
    </script>";
}

  public function callback_recommends_shop_cart() {
    if (is_cart()) {

      $user_id = get_current_user_id();

      // Use VisideaWoocommerce to send cart messages
      echo "<script> 
      var vs_timer_cart_1 = window.setInterval(function(){
        if (window.visideaWoocommerce != undefined) {
            window.clearInterval(vs_timer_cart_1);
            visideaWoocommerce.interaction('cart', 'cart', '".esc_js($user_id)."','');
        }
      }, 100);
      </script>";
  
      // Show recommendations
      echo "<script> 
      var vs_timer_cart_2 = window.setInterval(function(){
        if (window.visideaWoocommerce != undefined) {
            window.clearInterval(vs_timer_cart_2);
            visideaWoocommerce.recommend('cart');
        }
      }, 100);
      </script>";
    }
  }

}
